@@ -138,7 +138,6 @@ MakeMenu(char *name, HMENU editMenu, int editItem)
     return hmenu;
 }
 
-#ifdef XWIN_MULTIWINDOW
 /*
  * Callback routine that is executed once per window class.
  * Removes or creates custom window settings depending on LPARAM
@@ -197,7 +196,6 @@ ReloadEnumWindowsProc(HWND hwnd, LPARAM lParam)
 
     return TRUE;
 }
-#endif
 
 /*
  * Removes any custom icons in classes, custom menus, etc.
@@ -210,7 +208,6 @@ ReloadPrefs(winPrivScreenPtr pScreenPriv)
 {
     int i;
 
-#ifdef XWIN_MULTIWINDOW
     winScreenInfo *pScreenInfo = pScreenPriv->pScreenInfo;
 
     /* First, iterate over all windows, deleting their icons and custom menus.
@@ -221,7 +218,6 @@ ReloadPrefs(winPrivScreenPtr pScreenPriv)
      */
     if (pScreenInfo->fMultiWindow)
         EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, FALSE);
-#endif
 
     /* Now, free/clear all info from our prefs structure */
     for (i = 0; i < pref.menuItems; i++)
@@ -264,14 +260,12 @@ ReloadPrefs(winPrivScreenPtr pScreenPriv)
     g_hIconX = NULL;
     g_hSmallIconX = NULL;
 
-#ifdef XWIN_MULTIWINDOW
     if (pScreenInfo->fMultiWindow) {
         winInitGlobalIcons();
 
         /* Rebuild the icons and menus */
         EnumThreadWindows(g_dwCurrentThreadID, ReloadEnumWindowsProc, TRUE);
     }
-#endif
 
     /* Whew, done */
 }
@@ -385,14 +379,12 @@ HandleCustomWM_COMMAND(HWND hwnd, WORD command, winPrivScreenPtr pScreenPriv)
                         SetWindowPos(hwnd,
                                      HWND_TOPMOST,
                                      0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
-#if XWIN_MULTIWINDOW
                     {
                         winScreenInfo *pScreenInfo = pScreenPriv->pScreenInfo;
                         if (pScreenInfo->fMultiWindow)
                             /* Reflect the changed Z order */
                             winReorderWindowsMultiWindow();
                     }
-#endif
                     return TRUE;
 
                 case CMD_RELOAD:
@@ -409,7 +401,6 @@ HandleCustomWM_COMMAND(HWND hwnd, WORD command, winPrivScreenPtr pScreenPriv)
     return FALSE;
 }
 
-#ifdef XWIN_MULTIWINDOW
 /*
  * Add the default or a custom menu depending on the class match
  */
@@ -459,7 +450,6 @@ SetupSysMenu(HWND hwnd)
             MakeMenu(pref.defaultSysMenuName, sys, -1);
     }
 }
-#endif
 
 /*
  * Possibly add a menu to the toolbar icon
